@@ -7,13 +7,13 @@ extern "C" {
 #endif
 
 typedef struct {
-	PyObject_HEAD
-	PyObject *ob_ref;	/* Content of the cell or NULL when empty */
+    PyObject_HEAD
+    PyObject *ob_ref;       /* Content of the cell or NULL when empty */
 } PyCellObject;
 
 PyAPI_DATA(PyTypeObject) PyCell_Type;
 
-#define PyCell_Check(op) (Py_TYPE(op) == &PyCell_Type)
+#define PyCell_Check(op) Py_IS_TYPE(op, &PyCell_Type)
 
 PyAPI_FUNC(PyObject *) PyCell_New(PyObject *);
 PyAPI_FUNC(PyObject *) PyCell_Get(PyObject *);
